# Generated by Django 3.1.14 on 2023-01-25 11:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('outputs', '0012_auto_20230125_1252'),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='scheduler',
            name='custom_cron_string',
        ),
        migrations.AddConstraint(
            model_name='scheduler',
            constraint=models.CheckConstraint(check=models.Q(models.Q(('cron_string', ''), models.Q(_negated=True, routine='CUSTOM')), models.Q(models.Q(_negated=True, cron_string=''), ('routine', 'CUSTOM')), _connector='OR'), name='custom_cron_string'),
        ),
    ]
