# Generated by Django 3.1.14 on 2023-01-11 12:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('outputs', '0008_auto_20230111_1350'),
    ]

    operations = [
        migrations.AddField(
            model_name='export',
            name='exporter_path',
            field=models.CharField(blank=True, default='', max_length=100),
        ),
        migrations.AddField(
            model_name='scheduler',
            name='exporter_path',
            field=models.CharField(blank=True, default='', max_length=100),
        ),
    ]
