{#{% extends 'outputs/layout/scheduler.html' %}#}

{% load i18n crispy_forms_tags %}

{#{% block title %}{% if object %}{% trans 'Update scheduler' %}{% else %}{% trans 'Add scheduler' %}{% endif %}{% endblock %}#}

{#{% block content %}#}
    {{ block.super }}

    <div class="box-with-shadow p-5">
        {% crispy form %}
    </div>
{#{% endblock %}#}
