{% with id=widget.attrs.id %}
    {% for group, options, index in widget.optgroups %}
      {% if group %}
        <div>{{ group }}
          <div {% if id %} id="{{ id }}_{{ index }}"{% endif %}>
      {% endif %}
            {% for option in options %}
              <div>{% include option.template_name with widget=option %}</div>
            {% endfor %}
            {% if group %}
          </div>
        </div>
            {% endif %}
    {% endfor %}
{% endwith %}
