# Generated by Django 3.1.14 on 2022-02-08 18:58

from django.conf import settings
import django.contrib.postgres.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('outputs', '0006_auto_20200910_2149'),
    ]

    operations = [
        migrations.AlterField(
            model_name='export',
            name='fields',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=40), blank=True, default=None, null=True, size=None),
        )
    ]
