# Generated by Django 3.1.14 on 2023-01-11 12:50

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('outputs', '0007_auto_20220208_1958'),
    ]

    operations = [
        migrations.AlterField(
            model_name='scheduler',
            name='fields',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=40), blank=True, default=None, null=True, size=None),
        ),
    ]
