# Generated by Django 3.1.14 on 2023-01-25 11:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('outputs', '0011_reformat_export_fields_permissions'),
    ]

    operations = [
        migrations.AddField(
            model_name='scheduler',
            name='cron_string',
            field=models.CharField(blank=True, max_length=13),
        ),
        migrations.AddConstraint(
            model_name='scheduler',
            constraint=models.CheckConstraint(check=models.Q(('cron_string', ''), models.Q(_negated=True, routine='CUSTOM'), _connector='OR'), name='custom_cron_string'),
        )
    ]
